@@ -39,6 +39,7 @@ from _pulsar import (
     ConsumerCryptoFailureAction,
 )
 import pulsar
+from pulsar import _check_type
 
 class PulsarException(BaseException):
     """
@@ -220,6 +221,12 @@ class Producer:
         message was ever published.
         """
         return self._producer.last_sequence_id()
+    
+    def is_connected(self) -> bool:
+        """
+        Check if the producer is connected or not.
+        """
+        return self._producer.is_connected()
 
 class Consumer:
     """
@@ -320,6 +327,31 @@ class Consumer:
         )
         await future
 
+    async def negative_acknowledge(
+        self,
+        message: Union[pulsar.Message, pulsar.MessageId, _pulsar.Message, _pulsar.MessageId]
+    ) -> None:
+        """
+        Acknowledge the failure to process a single message asynchronously.
+
+        When a message is "negatively acked" it will be marked for redelivery after
+        some fixed delay. The delay is configurable when constructing the consumer
+        with {@link ConsumerConfiguration#setNegativeAckRedeliveryDelayMs}.
+        This call is not blocking.
+        
+        Parameters
+        ----------
+        message:
+            The received message or message id.
+        """
+        if isinstance(message, pulsar.Message):
+            msg = message._message
+        elif isinstance(message, pulsar.MessageId):
+            msg = message._msg_id
+        else:
+            msg = message
+        await asyncio.to_thread(self._consumer.negative_acknowledge, msg)
+
     async def unsubscribe(self) -> None:
         """
         Unsubscribe the current consumer from the topic asynchronously.
@@ -374,6 +406,26 @@ class Consumer:
         self._consumer.close_async(functools.partial(_set_future, future, value=None))
         await future
 
+    async def get_last_message_id(self) -> _pulsar.MessageId:
+        """
+        Asynchronously get the last message id.
+        """
+        future = asyncio.get_running_loop().create_future()
+        self._consumer.get_last_message_id_async(functools.partial(_set_future, future))
+        id = await future
+        return id
+
+    def redeliver_unacknowledged_messages(self):
+        """
+        Redelivers all the unacknowledged messages. In failover mode, the
+        request is ignored if the consumer is not active for the given topic. In
+        shared mode, the consumer's messages to be redelivered are distributed
+        across all the connected consumers. This is a non-blocking call and
+        doesn't throw an exception. In case the connection breaks, the messages
+        are redelivered after reconnect.
+        """
+        self._consumer.redeliver_unacknowledged_messages()
+
     def topic(self) -> str:
         """
         Return the topic this consumer is subscribed to.
@@ -724,6 +776,41 @@ class Client:
 
         schema.attach_client(self._client)
         return Consumer(await future, schema)
+    
+    def shutdown(self) -> None:
+        """
+        Shutdown the client and all the associated producers and consumers
+
+        Raises
+        ------
+        PulsarException
+        """
+        self._client.shutdown()
+
+    async def get_topic_partitions(self, topic: str) -> List[str]:
+        """
+        Get the list of partitions for a given topic in asynchronous mode.
+
+        If the topic is partitioned, this will return a list of partition names. If the topic is not partitioned, the returned list will contain the topic name itself.
+
+        This can be used to discover the partitions and create Reader, Consumer or Producer instances directly on a particular partition.
+
+        Parameters
+        ----------
+
+        topic: str
+            the topic name to lookup
+
+        Returns
+        -------
+        list
+            a list of partition names
+        """
+        _check_type(str, topic, 'topic')
+        future = asyncio.get_running_loop().create_future()
+        self._client.get_topic_partitions_async(topic, functools.partial(_set_future, future))
+        id = await future
+        return id
 
     async def close(self) -> None:
         """
