@@ -33,6 +33,7 @@ from unittest import (
 )
 
 import pulsar  # pylint: disable=import-error
+import _pulsar # pylint: disable=import-error
 from pulsar.asyncio import (  # pylint: disable=import-error
     Client,
     Consumer,
@@ -46,7 +47,29 @@ from pulsar.schema import (  # pylint: disable=import-error
     String,
 )
 
+from urllib.request import urlopen, Request
+
 SERVICE_URL = 'pulsar://localhost:6650'
+ADMIN_URL = "http://localhost:8080"
+TIMEOUT_MS = 10000  # Do not wait forever in tests
+
+def doHttpPost(url, data):
+    req = Request(url, data.encode())
+    req.add_header("Content-Type", "application/json")
+    urlopen(req)
+
+def doHttpPut(url, data):
+    try:
+        req = Request(url, data.encode())
+        req.add_header("Content-Type", "application/json")
+        req.get_method = lambda: "PUT"
+        urlopen(req)
+    except Exception as ex:
+        # ignore conflicts exception to have test idempotency
+        if "409" in str(ex):
+            pass
+        else:
+            raise ex
 
 class AsyncioTest(IsolatedAsyncioTestCase):
     """Test cases for asyncio Pulsar client."""
@@ -132,6 +155,47 @@ class AsyncioTest(IsolatedAsyncioTestCase):
         self.assertEqual(msg_id0.batch_index(), 0)
         self.assertEqual(msg_id1.batch_index(), 1)
 
+    async def test_get_topics_partitions(self):
+        topic_partitioned = "persistent://public/default/test_get_topics_partitions_async"
+        topic_non_partitioned = "persistent://public/default/test_get_topics_async_not-partitioned"
+
+        url1 = ADMIN_URL + "/admin/v2/persistent/public/default/test_get_topics_partitions_async/partitions"
+        doHttpPut(url1, "3")
+
+        self.assertEqual(
+            await self._client.get_topic_partitions(topic_partitioned),
+            [
+                "persistent://public/default/test_get_topics_partitions_async-partition-0",
+                "persistent://public/default/test_get_topics_partitions_async-partition-1",
+                "persistent://public/default/test_get_topics_partitions_async-partition-2",
+            ],
+        )
+        self.assertEqual(await self._client.get_topic_partitions(topic_non_partitioned), [topic_non_partitioned])
+
+    async def test_get_partitioned_topic_name(self):
+        url1 = ADMIN_URL + "/admin/v2/persistent/public/default/partitioned_topic_name_test/partitions"
+        doHttpPut(url1, "3")
+
+        partitions = [
+            "persistent://public/default/partitioned_topic_name_test-partition-0",
+            "persistent://public/default/partitioned_topic_name_test-partition-1",
+            "persistent://public/default/partitioned_topic_name_test-partition-2",
+        ]
+        self.assertEqual(
+            await self._client.get_topic_partitions("persistent://public/default/partitioned_topic_name_test"), partitions
+        )
+
+        consumer = await self._client.subscribe(
+            "persistent://public/default/partitioned_topic_name_test",
+            "partitioned_topic_name_test_sub",
+            consumer_type=pulsar.ConsumerType.Shared,
+        )
+        producer = await self._client.create_producer("persistent://public/default/partitioned_topic_name_test")
+        await producer.send(b"hello")
+
+        msg = await asyncio.wait_for(consumer.receive(), TIMEOUT_MS / 1000)
+        self.assertTrue(msg.topic_name() in partitions)
+
     async def test_create_producer_failure(self):
         try:
             await self._client.create_producer('tenant/ns/asyncio-test-send-failure')
@@ -156,6 +220,24 @@ class AsyncioTest(IsolatedAsyncioTestCase):
         except PulsarException as e:
             self.assertEqual(e.error(), pulsar.Result.AlreadyClosed)
 
+    async def test_producer_is_connected(self):
+        topic = f'asyncio-test-producer-is-connected-{time.time()}'
+        producer = await self._client.create_producer(topic)
+        self.assertTrue(producer.is_connected())
+        await producer.close()
+        self.assertFalse(producer.is_connected())
+
+    async def test_shutdown_client(self):
+        producer = await self._client.create_producer("persistent://public/default/partitioned_topic_name_test")
+        await producer.send(b"hello")
+        self._client.shutdown()
+
+        try:
+            await producer.send(b"hello")
+            self.fail("Expected AlreadyClosed exception after client shutdown")
+        except PulsarException as e:
+            self.assertEqual(e.error(), pulsar.Result.AlreadyClosed)
+
     async def _prepare_messages(self, producer: Producer) -> List[pulsar.MessageId]:
         msg_ids = []
         for i in range(5):
@@ -203,6 +285,36 @@ class AsyncioTest(IsolatedAsyncioTestCase):
         msg = await consumer.receive()
         self.assertEqual(msg.data(), b'msg-3')
 
+    async def test_consumer_negative_acknowledge(self):
+        topic = f'asyncio-test-consumer-negative-ack-{time.time()}'
+        sub = 'sub'
+        consumer = await self._client.subscribe(topic, sub,
+                                                consumer_type=pulsar.ConsumerType.Shared,
+                                                negative_ack_redelivery_delay_ms=100)
+        
+        producer = await self._client.create_producer(topic)
+        await self._prepare_messages(producer)
+        msgs = []
+        for _ in range(5):
+            msg = await consumer.receive()
+            msgs.append(msg)
+
+        await consumer.acknowledge(msgs[1])
+        await consumer.acknowledge(msgs[3])
+        
+        await consumer.negative_acknowledge(msgs[0])
+        await consumer.negative_acknowledge(msgs[2])
+        await consumer.negative_acknowledge(msgs[4])
+        await asyncio.sleep(0.2)
+        
+        received = []
+        for _ in range(3):
+            msg = await consumer.receive()
+            received.append(msg.data())
+        
+        self.assertEqual(sorted(received), [b'msg-0', b'msg-2', b'msg-4'])
+        await consumer.close()
+
     async def test_multi_topic_consumer(self):
         topics = ['asyncio-test-multi-topic-1', 'asyncio-test-multi-topic-2']
         producers = []
@@ -237,6 +349,59 @@ class AsyncioTest(IsolatedAsyncioTestCase):
         await verify_receive(consumer)
         await consumer.close()
 
+    async def test_consumer_get_last_message_id(self):
+        topic = f'asyncio-test-get-last-message-id-{time.time()}'
+        sub = 'sub'
+        consumer = await self._client.subscribe(topic, sub,
+                                                consumer_type=pulsar.ConsumerType.Shared)
+        producer = await self._client.create_producer(topic)
+        for i in range(5):
+            msg = f'msg-{i}'.encode()
+            await producer.send(msg)
+            last_msg_id = await consumer.get_last_message_id()
+            assert isinstance(last_msg_id, _pulsar.MessageId)
+            assert last_msg_id.entry_id() == i
+            await consumer.acknowledge(last_msg_id)
+        await consumer.close()
+
+    async def test_async_dead_letter_policy(self):
+        topic = f'asyncio-test-dlq-{time.time()}'
+        dlq_topic = 'dlq-' + topic
+        max_redeliver_count = 5
+
+        dlq_consumer = await self._client.subscribe(dlq_topic, "my-sub", consumer_type=pulsar.ConsumerType.Shared)
+        consumer = await self._client.subscribe(topic, "my-sub", consumer_type=pulsar.ConsumerType.Shared,
+                                    dead_letter_policy=pulsar.ConsumerDeadLetterPolicy(max_redeliver_count, dlq_topic, 'init-sub'))
+        producer = await self._client.create_producer(topic)
+
+        # Sen num msgs.
+        num = 10
+        for i in range(num):
+            await producer.send(b"hello-%d" % i)
+        await producer.flush()
+
+        # Redelivery all messages maxRedeliverCountNum time.
+        for i in range(1, num * max_redeliver_count + num + 1):
+            msg = await consumer.receive()
+            if i % num == 0:
+                consumer.redeliver_unacknowledged_messages()
+                print(f"Start redeliver msgs '{i}'")
+
+        with self.assertRaises(asyncio.TimeoutError):
+            await asyncio.wait_for(consumer.receive(), 0.1)
+
+        for i in range(num):
+            msg = await dlq_consumer.receive()
+            self.assertTrue(msg)
+            self.assertEqual(msg.data(), b"hello-%d" % i)
+            dlq_consumer.acknowledge(msg)
+
+        with self.assertRaises(asyncio.TimeoutError):
+            await asyncio.wait_for(dlq_consumer.receive(), 0.1)
+            
+        await consumer.close()
+        await dlq_consumer.close()
+
     async def test_unsubscribe(self):
         topic = f'asyncio-test-unsubscribe-{time.time()}'
         sub = 'sub'
